#!/bin/sh
#
# Component: dotcom
#
# Description:
#    Stores study objects resided on other HUB servers.
#
#    THIS SCRIPT IS INTENDED TO BE CALLED BY THE 'CLEANUP STUDY RESIDES ON MULTIPLE HUB SERVERS' TOOL ONLY!!!
#    THIS SCRIPT IS INTENDED TO RUN ON THE TARGET HUB SERVER ONLY!!!
#
#    The script places the images/reports copied from the other HUB servers to data/dicom
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh

Date=`date +"%Y%m%d%H%M%S"`

#
# Prints SRt files in the directory sorted by the number of predecessor document entries
# 
# $1 - directory the SRt files to be taken from
#
getSRtFiles_() {
	local dir="$1"

	[ -z "$dir" ] && return 1

	local srtFiles=$(ls $dir/ | grep ^SRt)
	[ -z "$srtFiles" ] && return 0

	local file=""
	local predecNum=""
	for file in $srtFiles ; do
		predecNum=$($DCMTK_HOME/bin/dcmdump +P "0008,1155" "$dir/$file" | wc -l)
		echo "$predecNum $file"
	done | sort -n | awk '{print $2}'
}

#
# Start StorescpReg task
#
# $1 - study
# $2 - file
# [$3] - tempFile (missing means reindex) 
#
startStorescpRegTask_() {
	local study="$1"
	local file="$2"
	local tempFile="$3"
	
	if [ -n "$tempFile" ] ; then
		$UTILS_HOME/bin/storeDICOMFile -f "${file}" -s "${study}" -t "$tempFile" -D "DELTA_STORESCP_REG_MULTIHUB"
		if [ $? -ne 0 ] ; then
			rm "$tempFile" 2>/dev/null
			return 1
		fi
	else
		$UTILS_HOME/bin/storeDICOMFile -r -f "${file}" -s "${study}" -D "DELTA_STORESCP_REG_MULTIHUB"
		return $?
	fi
	
	return 0
}

#
# Executes java command via Taskman
#
# $1 - command to be executed 
#
runjavacmd_() {
	local cmd="$1"
	[ -z "$cmd" ] && return 1

	$TASKD_HOME/runjavacmd -c "0 $cmd" -p "$JAVA_SERVER_PORT"
	return $?
}


#
# Prints usage information
#
# [$1] - error message to be printed first
#
usage_() {
	[ -n "$1" ] && echo "$1" >&2
	
	exit 1
}

#
# Cleans up working directory and exit
# 
# [$1] - error message to be printed
#
cleanup_() {
#	[ -d "$workDir" ] && rm -r "$workDir"
	
	if [ -n "$1" ] ; then
		echo "$1" 2>&1
		exit 1
	fi

	exit
}


#
# Sorting content of tar file received from a hub.
# Moves PbR, PbD and SRt objects to  $workDir/<hubnum>
# Moves study objects to $workDir/images
# Moves shortcuts to $workDir/shortcuts
#
# $1 - hubdir [subdirectory of $workDir]
# $2 - hub number [serial number for hubs]
#
sortTarContent_() {
	
	local hubdir="$1"
	local numHub="$2"
	local studyDir
	local shortcutDir
	local shortcutVar
	
	eval `sed 's;^\(.*\)=.*$;local \1;' "$workDir/$hubdir/CONTENT"`
	
	. "$workDir/$hubdir/CONTENT"
	[ $? -ne 0 ] && cleanup_ "CONTENT file could not be included for $hubdir"

	[ -z "$chmStudyDir" -o -z "$chmNumShortcuts" ] && cleanup_ "Invalid CONTENT file for $hubdir"

	# Move PbR, PbD and SRt objects to  $workDir/<hubnum>
	studyDir="$workDir/$hubdir/${chmStudyDir#/}"
	mv "$studyDir/SRt."*  "$studyDir/PbR."* "$studyDir/PbD."* "$workDir/$numHub" 2>/dev/null

	# Move study objects to $workDir/images
	find "$studyDir/" -type f -print0 | xargs -0 mv --target-directory="$workDir/images/"

	# Move shortcuts to $workDir/shortcuts
	for shortcutVar in ${!chmShortcut*}; do
		# skip variables not ending to _DIR
		[ "${shortcutVar%_DIR}" = "${shortcutVar}" ] && continue
		
		shortcutDir=`eval echo "\\${$shortcutVar}"`
		mv "$workDir/$hubdir/${shortcutDir#/}" "$workDir/shortcuts/"
	done
}


#
# Create shortcut directory and reindexes the PbR file
#
# $1 - path to shortcut in working directory
#
addShortcut_() {
	local shDir="$1"
	local shortcut=`basename "$shDir"`
	local pbrFileName=""

	eval `sed 's;^\(.*\)=;local sc\1=;' "$shDir/.info/storestate.rec"`
	
	[ -z "$scSourceStudy" ] && cleanup_ "Invalid Shortcut storestate.rec: No SourceStudy"
	
	pbrFileName=`ls $shDir/`
	
	[ -z "$pbrFileName" && cleanup_ "No PbR file found for shortcut"
	
	local storeStateArgs="-C --study=\"$shortcut\" --sourcestudy=\"$scSourceStudy\" --owner=yes"
	[ -n "$scProcessMode" ] && storeStateArgs=" $storeStateArgs --processmode=\"$scProcessMode\""
	[ -n "$scFolder" ] && storeStateArgs=" $storeStateArgs --folder=\"$scFolder\""
	
	# Create shortcut directory and storestate.rec file
	eval ${UTILS_HOME}/bin/storestate $storeStateArgs
	[ $? -ne 0 ] && cleanup_ "Shortcut directory could not be created" 
	 
	local shortcutDir=`$REPOSITORYHANDLER_HOME/bin/locateResource $REPOSITORY_DATA_DICOM "$shortcut"`
	[ -z "$shortcutDir" -o ! -d "$shortcutDir" ] && cleanup_ "Newly created shortcut directory could not located"

	# Move shortcut.cfg (if exists) to .info/ 
	[ -f "$shDir/.info/shortcut.cfg" ] && mv "$shDir/.info/shortcut.cfg" "$shortcutDir/.info/shortcut.cfg" 2>/dev/null
	
	# Add link_ reference to original (source) study
	echo ${UTILS_HOME}/bin/refcount -n "link_$shortcut" -t STUDYDB -i "$scSourceStudy" > $TMP/addref.cmd.$$
	${UTILS_HOME}/bin/refcount -n "link_$shortcut" -t STUDYDB -i "$scSourceStudy" 2>> $TMP/addref.cmd.$$
	echo $? >> $TMP/addref.cmd.$$

	tempFile="$IMAGE_TMP/${shortcut}_${pbrFileName}_$$"
	mv "$shDir/$pbrFileName" "${tempFile}"

	startStorescpRegTask_ "$shortcut" "$pbrFileName" "$tempFile"
}


newStudy="false"
study=""
reportFiles=""
dictationFiles=""
delReports=""
userId=""
workDir=""

while getopts 'nrs:u:w:' OPT; do
	case $OPT in
		n) newStudy="true" ;;
		r) delReports="true" ;;
		u) userId="$OPTARGS" ;;
		s) study="$OPTARG" ;;
		w) workDir="$OPTARG" ;;
		*) usage_ "Invalid parameter: $OPT" ;;
	esac
done
shift $(( $OPTIND - 1 ))

[ -z "$study" ] && usage_ "Missing required parameter: study"
[ -z "$workDir" ] && usage_ "Missing required parameter: working directory"
[ ! -d "$workDir" ] && usage_ "Invalid working directory: $workDir is not a directory"

. $CASES_HOME/includeStorestate.shi "$study"
[ $? -ne 0 -a "false" = "$newStudy" ] && cleanup_ "storestate.sh could not be read for the study $study"

studyDir=`$REPOSITORYHANDLER_HOME/bin/locateResource $REPOSITORY_DATA_DICOM $study`
[ $? -ne 0 -a "false" = "$newStudy" ] && cleanup_ "Study could not be located in the data repository"

# Check if archive server to be set for the study (it will be set later)
if [ "$ArchiveServer" = "$archiveServer" ] ; then
	# Archive Server need not be set
	archiveServer=""
elif [ -z "$archiveServer" ] ; then
	usage_ "Archive Server was not specified for the study $study"
fi
	
# Create working directory
mkdir "$workDir/images"
[ $? -ne 0 ] && cleanup_ "Working directory $workDir/images could not be created"
mkdir "$workDir/reports"
[ $? -ne 0 ] && cleanup_ "Working directory $workDir/reports could not be created"
mkdir "$workDir/shortcuts"
[ $? -ne 0 ] && cleanup_ "Working directory $workDir/shortcuts could not be created"


let numHubs=0
# DICOM object files received from other hubs are already untarred to working directory
while [ -n "$1" ] ; do
	[ ! -f "$workDir/$1/CONTENT" ] && cleanup_ "CONTENT file does not exist for $1"


	let numHubs+=1
	mkdir "$workDir/$numHubs"

	sortTarContent_ "$1" "$numHubs"
	shift
done

# Check images directory for report and dictation files
for file in `ls "$workDir/images/"` ; do
	if [ "${file#SRt.}" != "$file" ] ; then
		cleanup_ "report file $file remained amongst image objects"
	elif [ "${file#PbD.}" != "$file" ] ; then
		cleanup_ "report file $file remained amongst image objects"
	elif [ "${file#PbR.}" != "$file" ] ; then
		cleanup_ "report file $file remained amongst image objects"
	fi
done


# Reindex objects in study directory with -r specified to make sure all objects are in the archive
# Copy existing SRt,PbD objects to $workDir/reports

removeFiles=""
if [ -n "${studyDir}" ] ; then

	# Remove querymd5sum file to force study synchronization to CWL server
	[ -f "$studyDir/.info/querymd5sum" ] && rm -f "$studyDir/.info/querymd5sum"
	
	for file in `ls ${studyDir}`; do
		if [ "${file#SRt.}" != "$file" -o  "${file#PbD.}" != "$file" ] ; then
			# SRt or PbD file move it to workDir/reports or remove them based on settings
			if [ "$delReports" = "true" ] ; then
				removeFiles="$removeFiles -f \"$file\""
			else
				cp "${studyDir}/$file" "$workDir/reports"
				[ $? -ne 0 ] && cleanup_ "Report file $file could not be copied from ${studyDir}/$file to $workDir/reports/$file"
			fi
			continue
		fi
		
		startStorescpRegTask_ "$study" "$file"
	done
fi


# Fix report/dictation objects
latestSRt=""
mainPbD=""
if [ $numHubs -gt 0 ] ; then
	# move all SRt files to workdir/reports
	for rdir in reports $(seq 1 $numHubs) ; do
		srtList=$(getSRtFiles_ "$workDir/$rdir")
		[ -z "$srtList" ] && continue

		if [ -z "$latestSRt" ] ; then
			# Find the latest SRt in current directory and move SRt files to $workDir/reports
			for file in $srtList; do
				latestSRt="$file"
				if [ "$rdir" != reports ] ; then
					mv "$workDir/$rdir/$file" "$workDir/reports/$file"
					[ $? -ne 0 ] && cleanup_ "Report file $file could not be moved from $workDir/$rdir/$file to $workDir/reports/$file"
				fi
			done
		else
			# sort SRt files of study by number of predecessor document entries and add
			# predecessor documents from latest SRt. Move modified SRt file to $workDir/reports
			for file in $srtList; do
				if [ "$rdir" != reports ] ; then
					# the test above should always be true, but for safety
					$SR_HOME/bin/addSR "$workDir/reports/$latestSRt" "$workDir/$rdir/$file"
					[ $? -ne 0 ] && cleanup_ "Error adding $file to study as addendum"
					mv "$workDir/$rdir/$file" "$workDir/reports/$file"
					[ $? -ne 0 ] && cleanup_ "Report file $file could not be moved from $workDir/$rdir/$file to $workDir/reports/$file"
					latestSRt="$file"
				fi
			done
		fi
	done

	# merge all PbD files into workdir/reports/PbD
	pbdList=""
	for rdir in reports $(seq 1 $numHubs) ; do
		pbdFile=$(ls "$workDir/$rdir" | grep ^PbD)
		[ -z "$pbdFile" ] && continue

		# For the case there are multiple PbD files in the directory, though it's not a common case
		for file in $pbdFile; do
			if [ -z "$mainPbD" ] ; then
				if [ "$rdir" != "reports" ] ; then
					mv "$workDir/$rdir/$file" "$workDir/reports/$file"
					[ $? -ne 0 ] && cleanup_ "Dictation file $file could not be moved from $workDir/$rdir/$file to $workDir/reports/$file"
				fi
				mainPbD="$file"
				continue
			fi

			pbdList="$pbdList $workDir/$rdir/$file"
		done

		if [ -n "$pbdList" ] ; then
			$SR_HOME/bin/mergedictations "$workDir/reports/$mainPbD" $pbdList
			[ $? -ne 0 ] && cleanup_ "Error merging Dictation files"
		fi

	done
	
	# Find PbR to be kept - especially when study is being moved from another hub
	for rdir in reports $(seq 1 $numHubs) ; do
		pbrFile=$(ls "$workDir/$rdir" | grep ^PbR)
		[ -z "$pbrFile" ] && continue
		
		if [ -z "${studyDir}" ] ; then
			mv "$workDir/$rdir/$pbrFile" "$workDir/images/"
			break
		fi
	done	
fi

if [ "$delReports" = "true" -a -n "$removeFiles" ] ; then
	# Remove report files
	userParam=""
	[ -n "$userId" ] && userParam=" -u \"$userId\""

	runjavacmd_ "cases.RemoveFiles -s \"$study\"${userId}${removeFiles}"
	[ $? -ne 0 ] && cleanup_ "Report files could not be removed"
fi


# Place all image DICOM objects to study directory and start registering these objects
for file in $(ls $workDir/images) ; do
	tempFile="$IMAGE_TMP/${study}_${file}_$$"
	mv "$workDir/images/$file" "${tempFile}"
	startStorescpRegTask_ "$study" "$file" "$tempFile"
done

for file in $(ls $workDir/reports) ; do
	if [ "${file#PbD.}" != "$file" -a "$file" != "$mainPbD" ] ; then
		# PbD file but not the one to be kept
		continue
	elif [ "${file#PbR.}" != "$file" ] ; then
		# Do not keep PbR files
		continue
	fi

	tempFile="$IMAGE_TMP/${study}_${file}_$$"
	mv "$workDir/reports/$file" "${tempFile}"
	startStorescpRegTask_ "$study" "$file" "$tempFile"
done

# Place and Register shortcuts
for shortcut in $(ls  $workDir/shortcuts) ; do
	addShortcut_ "$workDir/shortcuts/$shortcut"
done
