#!/bin/sh
#
# Finds the route toward the Scheduler Module configured in the hierarchy
# and sends scheduler update message (generated by updateScheduler) to it.
#
# Usage:
#    sendSchedulerMessage.sh -f <filename> [-o <device>] [-x <excepthost>]
#
#        -f <filename>   : Scheduler update message file to be sent 
#        -o <device>     : Originating device (default: LOCAL)
#        -x <excepthost> : do not proliferate the file to this host
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh

LOGGER="/usr/bin/logger"
TAG="sendSchedulerMsg[$$]"

#
# Usage
#
usage_()
{
	if [ -n "$1" ] ; then
		echo $1
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "$1"
	fi
	cat <<- ENDOFUSAGE
		sendSchedulerMsg.sh -f <filename> -o <device> [-x <excepthost>]
	
		 -f <filename>   : Scheduler update message file to be sent and proliferated
		 -o <device>     : Originating device (default: LOCAL)
		 -x <excepthost> : do not proliferate the file to this host
	ENDOFUSAGE
} # usage


# proliferate.sh can be called directly:
# run as a task instead
if [ `basename $0` = "sendSchedulerMsg.sh" ] ; then
	$TASKD_HOME/runshellcmd.sh -R -r 0 $0 "$@"
	exit $?
fi

ERRMSG=""
FILENAME=""
DEVICE="LOCAL"
EXCEPT=""

EXCLUDE_DBONLY=""

$LOGGER -p $LOG_FACILITY.debug -t "$TAG" -- "args: $@"
while getopts 'f:o:x:' OPT; do
	case $OPT in
		f) FILENAME="$OPTARG" ;;
		o) DEVICE="$OPTARG" ;;
		x) EXCEPT="$OPTARG" ;;
		*) ERRMSG="Invalid_Parameter:$OPT" ;;
	esac
done

[ -z "$FILENAME" ] && ERRMSG="Missing Parameter"

rcid=`basename ${FILENAME}`
DATE=`date +"%Y%m%d%H%M%S"`


#override run method
run_()
{
	if [ -n "$ERRMSG" ] ; then
		usage_ "$ERRMSG"
		return 125
	fi

	# Create reference if file was received from parent/child
	[ -n "$EXCEPT" ] && $UTILS_HOME/bin/refcount -n "sendMsg" -i "$rcid" -t "SCHEDULERMSG" >>$LOG/refcount.error 2>&1
	
	[ -f "$IFACE_VAR/messageFlow.cfg" ] && . $IFACE_VAR/messageFlow.cfg

	if [ "$HasScheduler" != "true" ] ; then
		# No scheduler configured in the dotcom - nothing to do
		return 0
	fi

	if [ -n "$DeviceTowardScheduler" ] ; then
		if [ "$DeviceTowardScheduler" = "$EXCEPT" ] ; then
			# Circle - we received from the message from the device configured to have the scheduler
			# Do nothing
			return 0
		fi

		# upload the device toward the scheduler

		# Add "uploadto_" reference to -t SCHEDULERMSG -i `basename $FILENAME`
		# Reference will be removed by upload.sh after the message is successfully sent to peer device
		$UTILS_HOME/bin/refcount -n "uploadto_${DeviceTowardScheduler}" -i "$rcid" -t "SCHEDULERMSG" >>$LOG/refcount.error 2>&1

		$TASKD_HOME/createchildtask.sh -p "UPLOAD" -t "class" -c "dotcom.Upload -f \"$FILENAME\" -H ${DeviceTowardScheduler} -R SCHEDULERMSG -I \"$rcid\" -r $IFACE_HOME/bin/sendSchedulerMsg.sh -a -o -a $DEVICE"

		return 0
	fi
		

	# Scheduler is connected to the local server - do the call

	javaCmd="imagemed.scheduler.UpdateScheduler -f \"$FILENAME\""

	$TASKD_HOME/runjavacmd -c "0 $javaCmd"

	[ $? -ne 0 ] && return 1

	return 0
}

# override print_
print_()
{
	echo "Send messsage $FILENAME to Scheduler Module or toward the server having the Scheduler Module configured on"
}

#
# default get display name (for subjob)
#
get_display_name_()
{
	echo "Send message to scheduler module"
}

# override cleanup_
cleanup_()
{
	$UTILS_HOME/bin/refcount -r "sendMsg" -i "$rcid" -t "SCHEDULERMSG" >>$LOG/refcount.error 2>&1
}

